/******************************************************************************
 * The copyright in this software is being made available under the BSD        *
 * License, included below. This software may be subject to other third party  *
 * and contributor rights, including patent rights, and no such rights are     *
 * granted under this license.                                                 *
 *    <OWNER> = ISO                                                            *
 *    <ORGANIZATION> = ISO                                                     *
 *    <YEAR> = 2015                                                            *
 *    Copyright (c) 2012-2015, ISO                                             *
 *                                                                             *
 * All rights reserved.                                                        *
 * Redistribution and use in source and binary forms, with or without          *
 * modification, are permitted provided that the following conditions are met: *
 *       * Redistributions of source code must retain the above copyright      *
 *         notice,this list of conditions and the following disclaimer.        *
 *       * Redistributions in binary form must reproduce the above             *
 *         copyright notice, this list of conditions and the following         *
 *         disclaimer in the documentation and/or other materials provided     *
 *         with the distribution.                                              *
 *       * Neither the name of the <ORGANIZATION> nor the names of its         *
 *         contributors may be used to endorse or promote products derived     *
 *         from this software without specific prior written permission.       *
 *                                                                             *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" *
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE  *
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE   *
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR         *
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF        *
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS    *
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN     *
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)     *
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE  *
 * POSSIBILITY OF SUCH DAMAGE.                                                 *
 ******************************************************************************/

#ifndef _FILESTREAM_H_
#define _FILESTREAM_H_

/* This file implements output file stream. */

/* file stream implementation */
typedef struct jh_file_stream_ts {
/*  int (*read)(jh_stream_t* stream, unsigned char* buffer, size_t bytes);
    int (*write)(jh_stream_t* stream, unsigned char* buffer, size_t bytes);
    int (*seek)(jh_stream_t* stream, long long position, unsigned origin);
    jh_buf_mgr_t *pri; */
    jh_stream_t pub;  /* this exposes interface like above */
    FILE* stream;
    unsigned char *buf;
    size_t buf_size;
} jh_file_stream_t;

#define JH_FBUF_SIZE (0xfff)

static int jh_on_fstrm_init(jh_stream_t *s_in, unsigned char **buf, size_t *buf_size) {
    jh_file_stream_t *s = (jh_file_stream_t*)s_in;
    if (NULL == s) {
        return 0; /* invalid argument */
    }
    if (NULL != buf)
        *buf = s->buf;
    if (NULL != buf_size)
        *buf_size = s->buf_size;
    return 1;
}

static int jh_on_fstrm_empt(jh_stream_t *s_in, unsigned char **buf, size_t *buf_size)
{
    jh_file_stream_t *s = (jh_file_stream_t*)s_in;
    if (NULL == s) {
        return 0;
    }
   if (s_in->write(s_in, s->buf, s->buf_size) != s->buf_size) {
       return 0;
   }
   if (NULL != buf)
       *buf = s->buf;
   if (NULL != buf_size)
       *buf_size = s->buf_size;
  return 1;
}

static int jh_on_fstrm_term(jh_stream_t* s_in, size_t free_in_buffer) {
    /* this can be empty if data_size maintained by the writer */
    jh_file_stream_t *s = (jh_file_stream_t*)s_in;
    long int bytes = 0;
    if (NULL == s) {
        return 0; /* invalid arguments */
    }
    bytes = s->buf_size - free_in_buffer;
    if (bytes <0 || s_in->write(s_in, s->buf, bytes) != bytes) {
        return 0;
    }
    return 1;
}


static int jh_fstream_write(jh_stream_t* stream, unsigned char* buff, size_t bytes) {
    FILE* fp = ((jh_file_stream_t*)stream)->stream;
    size_t written = fwrite(buff, 1, bytes, fp);
    return (int) written;
}

static int jh_fstream_seek(jh_stream_t* stream, long long pos, unsigned origin) {
    FILE* fp = ((jh_file_stream_t*)stream)->stream;
    return fseek(fp, pos, origin) == 0;
}

/* jh_file_stream_t's FILE *stream must have been opened read/write for this to work */
static int jh_fstream_read(jh_stream_t* stream, unsigned char* buff, size_t bytes) {
    FILE* fp = ((jh_file_stream_t*)stream)->stream;
    size_t read = fread(buff, 1, bytes, fp);
    return (int) read;
}


#endif  /* _FILESTREAM_H_ */

